//
//  PUColorPicker.m
//  BJPlayerManagerUI_Example
//
//  Created by xijia dai on 2020/8/3.
//  Copyright © 2020 oushizishu. All rights reserved.
//

#import "PUColorPicker.h"
#import <BJLiveBase/BJLiveBase.h>

@interface PUColorPicker ()

@property (nonatomic) CGFloat saturationExponentTop;
@property (nonatomic) NSInteger elementSize;
@property (nonatomic) UIView *indicatorView;

@end

@implementation PUColorPicker

- (instancetype)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
        self.saturationExponentTop = 2.0;
        self.elementSize = 1.0;
        [self setupTapGesture];
    }
    return self;
}

#pragma mark - draw rect

/**
 通过色调 hue 饱和度 saturation 亮度 brightness 绘制矩形取色盘
 从上到下，饱和度从 0 -> 1 -> 0，亮度从 1 -> 0，从左到右，色调 0 -> 1 实现
 如果绘制圆形取色盘，通过正切值计算的弧度的变化改变色调，通过计算平方和获得的点的圆心距来改变饱和度
 参考的 @see https://stackoverflow.com/questions/27208386/simple-swift-color-picker-popover-ios 的实现
 除此之外，也可以通过给定一张含有全部颜色图片，直接获取指定点的颜色即可
 */
- (void)drawRect:(CGRect)rect {
    [self drawRectColorWithRect:rect];
}

- (void)drawEclipseColorWithRect:(CGRect)rect {
    CGFloat width = rect.size.width;
    CGFloat height = rect.size.height;
    CGFloat center = 0.5;
    CGContextRef context = UIGraphicsGetCurrentContext();
    for (NSInteger y1 = 0; y1 < height; y1 += self.elementSize) {
        for (NSInteger x1 = 0; x1 < width; x1 += self.elementSize) {
            CGFloat x = x1 / width;
            CGFloat y = y1 / height;
            CGFloat saturation = MIN(0.5, hypot(x - center, y - center)) * 2;
            CGFloat angle = -atan2(y - center, x - center);
            CGFloat r = remainder(angle, 2 * M_PI);
            CGFloat value = angle < 0.0 ? r + 2 * M_PI : r;
            CGFloat hue = value / (2 * M_PI);
            UIColor *color = [UIColor colorWithHue:hue saturation:saturation brightness:1.0 alpha:1.0];
            CGContextSetFillColorWithColor(context, color.CGColor);
            CGContextFillRect(context, CGRectMake(x1, y1, self.elementSize, self.elementSize));
        }
    }
}

- (void)drawRectColorWithRect:(CGRect)rect {
    CGFloat width = rect.size.width;
    CGFloat height = rect.size.height;
    CGContextRef context = UIGraphicsGetCurrentContext();
    for (NSInteger y = 0; y < height; y += self.elementSize) {
        BOOL half = y < (height / 2.0);
        CGFloat saturation = half ? 2.0 * y / height : 2.0 * (height - y) / height;
        saturation = half ? powf(saturation,  self.saturationExponentTop) : 1.0;
        CGFloat brightness = half ? 1.0 : 2.0 * (height - y) / height;
        for (NSInteger x = 0; x < width; x += self.elementSize) {
            CGFloat hue = x / width;
            UIColor *color = [UIColor colorWithHue:hue saturation:saturation brightness:brightness alpha:1.0];
            CGContextSetFillColorWithColor(context, color.CGColor);
            CGContextFillRect(context, CGRectMake(x, y, self.elementSize, self.elementSize));
        }
    }
}

- (void)layoutSubviews {
    [super layoutSubviews];
    [self updateIndicatorWithPoint:[self pointWithColor:self.selectedColor]];
}

#pragma mark - gesture

- (void)setupTapGesture {
    self.userInteractionEnabled = YES;
    bjl_weakify(self);
    UITapGestureRecognizer *gesture = [UITapGestureRecognizer bjl_gestureWithHandler:^(__kindof UIGestureRecognizer * _Nullable gesture) {
        bjl_strongify(self);
        CGPoint point = [gesture locationInView:self];
        UIColor *color = [self colorWithPoint:point];
        self.selectedColor = color;
    }];
    [self addGestureRecognizer:gesture];
    UIPanGestureRecognizer *panGesture = [UIPanGestureRecognizer bjl_gestureWithHandler:^(__kindof UIGestureRecognizer * _Nullable gesture) {
        bjl_strongify(self);
        CGPoint point = [gesture locationInView:self];
        UIColor *color = [self colorWithPoint:point];
        self.selectedColor = color;
    }];
    [self addGestureRecognizer:panGesture];
    [gesture requireGestureRecognizerToFail:panGesture];
}

#pragma mark - indicator

- (void)updateIndicatorWithPoint:(CGPoint)point {
    CGFloat size = 8.0;
    CGFloat x = point.x - size / 2.0;
    CGFloat y = point.y - size / 2.0;
    self.indicatorView.frame = CGRectMake(x, y, size, size);
}

#pragma mark - setter

- (void)setSelectedColor:(UIColor *)selectedColor {
    _selectedColor = selectedColor;
    [self updateIndicatorWithPoint:[self pointWithColor:selectedColor]];
}

#pragma mark - getter

- (UIColor *)colorWithPoint:(CGPoint)point {
    CGFloat x = self.elementSize * (NSInteger)(point.x / self.elementSize);
    CGFloat y = self.elementSize * (NSInteger)(point.y / self.elementSize);
    CGFloat width = self.bounds.size.width;
    CGFloat height = self.bounds.size.height;
    BOOL half = y < (height / 2.0);
    CGFloat saturation = half ? 2.0 * y / height : 2.0 * (height - y) / height;
    saturation = half ? powf(saturation,  self.saturationExponentTop) : 1.0;
    CGFloat brightness = half ? 1.0 : 2.0 * (height - y) / height;
    CGFloat hue = x / width;
    UIColor *color = [UIColor colorWithHue:hue saturation:saturation brightness:brightness alpha:1.0];
    return color;
}

- (CGPoint)pointWithColor:(UIColor *)color {
    CGFloat hue = 0.0;
    CGFloat saturation = 0.0;
    CGFloat brightness = 0.0;
    CGFloat width = self.bounds.size.width;
    CGFloat height = self.bounds.size.height;
    BOOL success = [color getHue:&hue saturation:&saturation brightness:&brightness alpha:nil];
    if (!success) {
        return CGPointZero;
    }
    CGFloat y = 0.0;
    CGFloat halfHeight = height / 2.0;
    if (brightness >= 0.99) {
        CGFloat percentageY = powf(saturation, 1.0 / self.saturationExponentTop);
        y = MIN(MAX(0, percentageY * halfHeight), height);
    }
    else {
        y = MIN(MAX(0, halfHeight + halfHeight * (1.0 - brightness)), height);
    }
    CGFloat x = MIN(MAX(0, hue * width), width);
    return CGPointMake(x, y);
}

- (NSString *)hexStringWithColor:(UIColor *)color {
    CGFloat red = 0.0;
    CGFloat green = 0.0;
    CGFloat blue = 0.0;
    BOOL succes = [color getRed:&red green:&green blue:&blue alpha:nil];
    if (!succes) {
        return @"#FFFFF";
    }
    NSString *hexString = [NSString stringWithFormat:@"#%02lX%02lX%02lX", (long)(red * 255), (long)(green * 255), (long)(blue * 255)];
    return hexString;
}

- (UIView *)indicatorView {
    if (!_indicatorView) {
        _indicatorView = ({
            UIView *view = [UIView new];
            CAShapeLayer *layer = [CAShapeLayer layer];
            UIBezierPath *trianglePath = [UIBezierPath bezierPath];
            [trianglePath moveToPoint:CGPointMake(0.0, 0.0)];
            [trianglePath addLineToPoint:CGPointMake(8.0, 0.0)];
            [trianglePath addLineToPoint:CGPointMake(4.0, 8.0)];
            [trianglePath closePath];
            layer.path = trianglePath.CGPath;
            layer.fillColor = [UIColor blackColor].CGColor;
            [view.layer addSublayer:layer];
            [self addSubview:view];
            view;
        });
    }
    return _indicatorView;
}

@end
